import sys
import os
import unittest
import json

# Add the parent directory to the sys.path
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from html_to_delta import html_to_delta

class TestHtmlToDelta(unittest.TestCase):
    
    def setUp(self):
        pass
    
    def test_plain_text(self):
        html_content = "<p>This is plain text.</p>"
        expected_output = {
            "ops": [
                {"insert": "This is plain text."},
                {"insert": "\n", "attributes": {"block": "paragraph"}}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)
    
    def test_bold_text(self):
        html_content = "<p>This is <strong>bold</strong> text.</p>"
        expected_output = {
            "ops": [
                {"insert": "This is "},
                {"insert": "bold", "attributes": {"bold": True}},
                {"insert": " text."},
                {"insert": "\n", "attributes": {"block": "paragraph"}}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)
    
    def test_link_text(self):
        html_content = '<p>This is a <a href="https://example.com">link</a>.</p>'
        expected_output = {
            "ops": [
                {"insert": "This is a "},
                {"insert": "link", "attributes": {"link": "https://example.com"}},
                {"insert": "."},
                {"insert": "\n", "attributes": {"block": "paragraph"}}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)
    
    def test_blockquote(self):
        html_content = "<blockquote>This is a blockquote.</blockquote>"
        expected_output = {
            "ops": [
                {"insert": "This is a blockquote."},
                {"insert": "\n", "attributes": {"blockquote": True}}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)
    
    def test_table(self):
        html_content = """
        <table>
            <tr><th>Header 1</th><th>Header 2</th></tr>
            <tr><td>Row 1, Cell 1</td><td>Row 1, Cell 2</td></tr>
            <tr><td>Row 2, Cell 1</td><td>Row 2, Cell 2</td></tr>
        </table>
        """
        expected_output = {
            "ops": [
                {"insert": "\n"},
                {"insert": {"table": [
                    ["Header 1", "Header 2"],
                    ["Row 1, Cell 1", "Row 1, Cell 2"],
                    ["Row 2, Cell 1", "Row 2, Cell 2"]
                ]}},
                {"insert": "\n"}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)
    
    def test_div(self):
        html_content = """
        <div>This is a div.</div>
        """
        expected_output = {
            "ops": [
                {"insert": "\n"},
                {"insert": "This is a div."},
                {"insert": "\n"}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)
    
    def test_combined_formatting(self):
        html_content = """
        <p>This is <strong>bold</strong>, <em>italic</em>, and <u>underlined</u> text.</p>
        """
        expected_output = {
            "ops": [
                {"insert": "\n"},
                {"insert": "This is "},
                {"insert": "bold", "attributes": {"bold": True}},
                {"insert": ", "},
                {"insert": "italic", "attributes": {"italic": True}},
                {"insert": ", and "},
                {"insert": "underlined", "attributes": {"underline": True}},
                {"insert": " text."},
                {"insert": "\n", "attributes": {"block": "paragraph"}},
                {"insert": "\n"}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)
    
    def test_image(self):
        html_content = '<p>This is an <img src="https://example.com/image.jpg"> image.</p>'
        expected_output = {
            "ops": [
                {"insert": "This is an "},
                {"insert": {"image": "https://example.com/image.jpg"}},
                {"insert": " image."},
                {"insert": "\n", "attributes": {"block": "paragraph"}}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)
    
    def test_nested_elements(self):
        html_content = """
        <p>This is a <strong>bold <em>and italic</em></strong> text.</p>
        """
        expected_output = {
            "ops": [
                {"insert": "\n"},
                {"insert": "This is a "},
                {"insert": "bold ", "attributes": {"bold": True}},
                {"insert": "and italic", "attributes": {"bold": True, "italic": True}},
                {"insert": " text."},
                {"insert": "\n", "attributes": {"block": "paragraph"}},
                {"insert": "\n"}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)
    
    def test_color_text(self):
        html_content = '<p>This is <span style="color: red;">red</span> text.</p>'
        expected_output = {
            "ops": [
                {"insert": "\n"},
                {"insert": "This is "},
                {"insert": "red", "attributes": {"color": "#ff0000"}},
                {"insert": " text."},
                {"insert": "\n", "attributes": {"block": "paragraph"}},
                {"insert": "\n"}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)

    
    def test_strikethrough_text(self):
        html_content = '<p>This is <del>strikethrough</del> text.</p>'
        expected_output = {
            "ops": [
                {"insert": "\n"},
                {"insert": "This is "},
                {"insert": "strikethrough", "attributes": {"strike": True}},
                {"insert": " text."},
                {"insert": "\n", "attributes": {"block": "paragraph"}},
                {"insert": "\n"}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)
    
    def test_font_text(self):
        html_content = '<p>This is <span style="font-family: Arial, sans-serif;">Arial</span> text.</p>'
        expected_output = {
            "ops": [
                {"insert": "\n"},
                {"insert": "This is "},
                {"insert": "Arial", "attributes": {"font": "Arial, sans-serif"}},
                {"insert": " text."},
                {"insert": "\n", "attributes": {"block": "paragraph"}},
                {"insert": "\n"}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)
    
    def test_indentation(self):
        html_content = '<p style="text-indent: 20px;">This is indented text.</p>'
        expected_output = {
            "ops": [
                {"insert": "\n"},
                {"attributes": {"indent": 20}, "insert": "This is indented text."},
                {"insert": "\n", "attributes": {"block": "paragraph"}},
                {"insert": "\n"}
            ]
        }
        result = html_to_delta(html_content)
        self.assertEqual(json.loads(result), expected_output)
    
    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



from bs4 import BeautifulSoup, NavigableString, Tag
import json

class CustomDelta:
    def __init__(self):
        self.ops = []

    def insert(self, text, attributes=None):
        if attributes:
            self.ops.append({'insert': text, 'attributes': attributes})
        else:
            self.ops.append({'insert': text})

    def to_dict(self):
        return {'ops': self.ops}

def parse_element(element, delta):
    try:
        if isinstance(element, NavigableString):
            delta.insert(str(element))
        elif isinstance(element, Tag):
            if element.name == 'p':
                parse_children(element, delta)
                delta.insert('\n', {'block': 'paragraph'})
            elif element.name in ['strong', 'b']:
                parse_children(element, delta, {'bold': True})
            elif element.name == 'em':
                parse_children(element, delta, {'italic': True})
            elif element.name == 'u':
                parse_children(element, delta, {'underline': True})
            elif element.name == 'a':
                href = element.get('href', '')
                parse_children(element, delta, {'link': href})
            elif element.name == 'h1':
                parse_children(element, delta)
                delta.insert('\n', {'header': 1})
            elif element.name == 'h2':
                parse_children(element, delta)
                delta.insert('\n', {'header': 2})
            elif element.name == 'h3':
                parse_children(element, delta)
                delta.insert('\n', {'header': 3})
            elif element.name == 'ol':
                parse_children(element, delta)
                delta.insert('\n', {'list': 'ordered'})
            elif element.name == 'ul':
                parse_children(element, delta)
                delta.insert('\n', {'list': 'bullet'})
            elif element.name == 'li':
                parse_children(element, delta)
                delta.insert('\n', {'list-item': True})
            elif element.name == 'br':
                delta.insert('\n')
            elif element.name == 'img':
                src = element.get('src', '')
                delta.insert({'image': src})
            elif element.name == 'iframe':
                src = element.get('src', '')
                delta.insert({'iframe': src})
            elif element.name == 'table':
                parse_table(element, delta)
            elif element.name == 'blockquote':
                parse_children(element, delta)
                delta.insert('\n', {'blockquote': True})
            elif element.name == 'div':
                parse_children(element, delta)
            elif element.name == 'span':
                parse_children(element, delta)
            elif element.name == 'code':
                parse_children(element, delta)
                delta.insert('\n', {'code': True})
            elif element.name == 'pre':
                parse_children(element, delta)
                delta.insert('\n', {'code-block': True})
            elif element.name == 'sup':
                parse_children(element, delta)
                delta.insert('\n', {'script': 'super'})
            elif element.name == 'sub':
                parse_children(element, delta)
                delta.insert('\n', {'script': 'sub'})
            else:
                parse_children(element, delta)
    except Exception as e:
        print(f"Error parsing element: {e}")

def parse_children(element, delta, attributes=None):
    try:
        for child in element.children:
            parse_element(child, delta if not attributes else CustomDeltaWithAttributes(delta, attributes))
    except Exception as e:
        print(f"Error parsing children: {e}")

def parse_table(table, delta):
    try:
        table_content = []
        for row in table.find_all('tr'):
            row_content = []
            for cell in row.find_all(['td', 'th']):
                cell_text = ''.join(cell.stripped_strings)
                row_content.append(cell_text)
            table_content.append(row_content)
        delta.insert({'table': table_content})
    except Exception as e:
        print(f"Error parsing table: {e}")

class CustomDeltaWithAttributes(CustomDelta):
    def __init__(self, delta, attributes):
        super().__init__()
        self.delta = delta
        self.attributes = attributes

    def insert(self, text, attributes=None):
        try:
            combined_attributes = self.attributes.copy()
            if attributes:
                combined_attributes.update(attributes)
            self.delta.insert(text, combined_attributes)
        except Exception as e:
            print(f"Error inserting text with attributes: {e}")

def html_to_delta(html):
    try:
        soup = BeautifulSoup(html, 'html.parser')
        
        # Check if body tag is present, otherwise wrap content in a div
        if not soup.body:
            body = soup.new_tag('body')
            soup.insert(0, body)
            body.append(BeautifulSoup(html, 'html.parser'))
        
        delta = CustomDelta()
        for element in soup.body.children:
            parse_element(element, delta)
        return json.dumps(delta.to_dict(), indent=2)
    except Exception as e:
        print(f"Error converting HTML to Delta: {e}")

# Example usage:
html_content = """
<p style="text-indent: 20px;">This is indented text.</p>
"""

delta_format = html_to_delta(html_content)
print(delta_format)



from setuptools import setup, find_packages

setup(
    name='html_to_quill_delta',
    version='0.1.0',
    description='Convert HTML content to Quill Delta format',
    author='patel-world',
    author_email='patel.dp2024@gmail.com',
    url='https://github.com/Patel-world/html_to_quill_delta',
    packages=find_packages(),
    install_requires=[
        'beautifulsoup4',
        'quill-delta'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)
